#include "visorarbol.h"
#include "operaciondibujo.h"
#include "dibujarnodos.h"
#include "localizarnodo.h"
#include "arbolbinarioordenado.h"
#include "nodobinario.h"
#include "extensionarbol.h"

#include <qrect.h>
#include <qpainter.h>
#include <math.h>


//! Construye un visor de rbol

/*!
    Construye un widget visualizador de rboles binarios.

    \param abo Arbol a visualizar.
    \param parent Widget padre.
    \param name Nombre del widget.
    \param f Opciones del widget.
 */
VisorArbol::VisorArbol ( ArbolBinarioOrdenado *abo, QWidget * parent = 0,
    const char * name = 0, WFlags f = 0 ):QScrollView (parent, name, f )
{
    arbolModificado(abo);
}

//! Destruye un visor de rbol

/*!
    Destruye un widget visualizador de rboles binarios.
 */
VisorArbol::~VisorArbol()
{

}

//! El rbol a visualizar ha sido modificado

/*!
    Indica al widget que el rbol a visualizar <tt>abo</tt>
    se ha modificado (o es uno nuevo).

    \param abo rbol a visualizar.
 */
void VisorArbol::arbolModificado( ArbolBinarioOrdenado *abo )
{
    _arbol = abo;
    _nodoSeleccionado = NULL;

    if (!_arbol) {
        resizeContents( 0, 0 );
    } else {
        if ( _arbol->vacio() ) {
            resizeContents( 0, 0 );
        } else {
            ExtensionArbol op;
            int xIni=0;
            recorre( _arbol->raiz(), 0, xIni, &op );
            resizeContents( op.maxX()+_anchuraNodo, op.maxY()+_alturaNodo );
        }
    }
    updateContents(0,0,visibleWidth(),visibleHeight());
}

//! Dibuja los contenidos de la vista

/*!
    Actualiza el contenido de la vista dentro del rectngulo
    de clipping dado.

    \param p Objeto dibujante.
    \param clipx Coordenada x de la esquin superior izquierda
        del rectnculo de recorte.
    \param clipy Coordenada y de la esquin superior izquierda
        del rectnculo de recorte.
    \param clipw Anchura del rectngulo de recorte.
    \param cliph Anltura del rectngulo de recorte.
*/
void VisorArbol::drawContents ( QPainter * p, int clipx,
            int clipy, int clipw, int cliph )
{
    QRect recorte( clipx, clipy, clipx+clipw, clipy+cliph );

    QBrush brocha(  qRgb(0xf7, 0xef, 0xd6), Qt::SolidPattern );
    p->fillRect( recorte, brocha );

    if (!_arbol)
        return;

    int xIni=_espacioHorizontal;
    /*if ( _arbol->raiz() )
        dibujaArbol(p, _arbol->raiz(), recorte, 0, xIni);*/
    if ( _arbol->raiz() ) {
        DibujarNodos operacion( p, recorte);
        recorre( _arbol->raiz(), 0, xIni, &operacion);
    }

}

//! Acciones a realizar para eventos de ratn

/*!
    \param ev Evento de ratn.
 */
void VisorArbol::contentsMousePressEvent ( QMouseEvent *ev )
{
    if (ev->button()!=LeftButton)
        return;

    int x=ev->x();
    int y=ev->y();

    NodoBinario *anterior = _nodoSeleccionado;
    _nodoSeleccionado = NULL;
    int xIni = _espacioHorizontal;

    LocalizarNodo op(x,y);
    recorre( _arbol->raiz(), 0, xIni, &op);

    _nodoSeleccionado = op.encontrado();

    if (anterior!=_nodoSeleccionado) {
        updateContents( 0, 0, visibleWidth(), visibleHeight() );
        emit( nodoSeleccionado( _nodoSeleccionado ) );
    }
}

//! Recorre el rbol aplicando una operacin de dibujo.

/*!
    Recorre el subrbol de raiz <tt>nodo</tt> aplicando la
    operacin de dibujo <tt>op</tt>.

    \param nodo Nodo actual.
    \param prof Profundidad del nodo.
    \param xIni Coordenada x inicial actual.
    \param op Operacin de dibujo a aplicar.

    \returns Caja que inscribe al nodo.
*/

QRect VisorArbol::recorre( NodoBinario *nodo, int prof, int &xIni,
    OperacionDibujo *op)
{
    QRect cajaNodo;
    QRect cajaIzquierda, cajaDerecha;

    cajaNodo.setTop( (1+prof)*(_alturaNodo+_espacioVertical) );
    cajaNodo.setBottom( cajaNodo.top()+_alturaNodo );

    if ( nodo->esHoja() ) {
        cajaNodo.setLeft(xIni);
        cajaNodo.setRight(xIni+_anchuraNodo);

        xIni += _anchuraNodo;
    } else {
        if ( nodo->hijoIzquierdo() ) {
            cajaIzquierda=recorre( nodo->hijoIzquierdo(), prof+1, xIni, op );
        }
        xIni+=_espacioHorizontal;

        if (nodo->hijoDerecho()) {
            cajaDerecha=recorre( nodo->hijoDerecho(), prof+1, xIni, op );
        }
        xIni+=_espacioHorizontal;

        xIni+=_espacioHorizontal;

        if (!nodo->hijoIzquierdo())
            cajaNodo.setLeft( cajaDerecha.left() - _espacioHorizontal );
        else if (!nodo->hijoDerecho())
            cajaNodo.setLeft( cajaIzquierda.left() + _espacioHorizontal );
        else {
            cajaNodo.setLeft( (cajaDerecha.left() + cajaIzquierda.left()) / 2 );
        }
        cajaNodo.setRight( cajaNodo.left() + _anchuraNodo );
    }

    if ( nodo == _nodoSeleccionado )
        op->aplicar( cajaNodo, cajaIzquierda, cajaDerecha, nodo, true);
    else
        op->aplicar( cajaNodo, cajaIzquierda, cajaDerecha, nodo, false);

    return cajaNodo;
}
